// Copyright 2016 The Xorm Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package builder

import "fmt"

// Match defines full-text match condition
type Match [2]string

var _ Cond = Match{"", ""}

// WriteTo write SQL to Writer
func (m Match) WriteTo(w Writer) error {
	if _, err := fmt.Fprintf(w, "to_tsvector(%s) @@ to_tsquery(?)", m[0]); err != nil {
		return err
	}
	w.Append(m[1])
	return nil
}

// And implements And with other conditions
func (m Match) And(conds ...Cond) Cond {
	return And(m, And(conds...))
}

// Or implements Or with other conditions
func (m Match) Or(conds ...Cond) Cond {
	return Or(m, Or(conds...))
}

// IsValid tests if this condition is valid
func (m Match) IsValid() bool {
	return len(m[0]) > 0 && len(m[1]) > 0
}
